#!/usr/bin/env python

import os
import sys
import gtk
import shutil
from subprocess import Popen, PIPE

import gettext
from gettext import gettext as _
gettext.textdomain('xdiagnose')

from xdiagnose import utils

def dbg(msg):
    sys.stderr.write("%s\n" %(msg))

def execute(cmd):
    '''Shells out and executes the command, returning error code'''
    proc = Popen(cmd, stdout=PIPE, stdin=PIPE, shell=True)
    stdout, stderr = proc.communicate()
    return proc.returncode


class XDiagnoseApplication(object):
    def __init__(self):
        self.xorg_conf_path = "/etc/X11/xorg.conf"
        self.xorg_conf_d_path = "/etc/X11/xorg.conf.d"
        self.xorg_conf_backup_path = "%s-backup" %(xorg_conf)
        self.xorg_conf_d_backup_path = "%s-backup" %(xorg_conf_d)
        self.current_page = None

        # Top level UI containers
        self.window = self._create_top_window()
        self.frame = self._create_top_frame()

        # Create UI elements
        self.accelerators = gtk.AccelGroup()
        self.pages = {
            'welcome': self._create_welcome_page(),
            'actions': self._create_actions_page(),
            'reconfigure': self._create_reconfigure_page(),
            'troubleshoot': self._create_troubleshoot_page()
            }

        # Assemble the UI
        self.window.add(self.frame)

        # Positions
        self.window.set_default_size(400, 300)
        self.window.set_size_request(400, 300)

        # Refresh
        self.update_frame()
        self.window.show_all()
        self.on_page(None, 'welcome')

    def _create_top_window(self):
        win = gtk.Window(gtk.WINDOW_TOPLEVEL)
        win.connect('delete_event', self.on_delete)
        win.set_border_width(10)
        return win

    def _create_top_frame(self):
        frame = gtk.Frame("")
        self.page_title = gtk.Label("")
        frame.set_label_widget(self.page_title)
        frame.set_label_align(0.0, 0.0)
        return frame

    def _create_button(self, primary_text, secondary_text):
        button = gtk.Button(primary_text)
        button_label = button.get_children()[0]
        button_label.set_markup(
            "<b>%s</b>\n<small>%s</small>"
            %(primary_text, secondary_text))
        button_label.set_alignment(0.0, 0.0)
        button.set_relief(gtk.RELIEF_NONE)
        button.show()
        return button

    def _create_nav_bar(self, next_page=None, prev_page=None):
        hbox_navigation = gtk.HButtonBox()

        if prev_page is not None:
            hbox_navigation.set_property("layout-style", gtk.BUTTONBOX_START)
            prev_button = gtk.Button("Back")
            prev_button.connect("clicked", self.on_page, prev_page)
            hbox_navigation.pack_start(prev_button)
        if next_page is not None:
            hbox_navigation.set_property("layout-style", gtk.BUTTONBOX_END)
            next_button = gtk.Button("Next")
            next_button.connect("clicked", self.on_page, next_page)
            hbox_navigation.pack_end(next_button)
        return hbox_navigation

    def _create_page(self, text):
        page = gtk.VBox(spacing=5)
        page.set_border_width(10)
        label = gtk.Label(text)
        label.set_line_wrap(True)
        label.set_use_markup(True)
        label.set_alignment(0.0, 0.0)
        page.pack_start(label, expand=False)
        return page

    def _create_welcome_page(self):
        page = self._create_page(
            '<big>Ubuntu is running in low-graphics mode</big>\n\n' +
            'Your graphics and input devices could not be detected correctly.  You will need to configure these yourself.')
        nav_bar = self._create_nav_bar(next_page="actions")
        page.pack_end(nav_bar, expand=False)
        return page

    def _create_actions_page(self):
        page = self._create_page("What would you like to try?")

        hbox = gtk.HBox()

        button1 = self._create_button("Run in low-graphics mode",
                                      "Use low-graphics for just one session")
        button1.connect("clicked", self.on_low_res_mode)
        button2 = self._create_button("Reconfigure",
                                      "Try to automatically fix the problem")
        button2.connect("clicked", self.on_page, "reconfigure")
        button3 = self._create_button("Troubleshoot",
                                      "Review errors, configs, and logs")
        button3.connect("clicked", self.on_page, "troubleshoot")
        button4 = self._create_button("Console",
                                      "Exit to the console login")
        button4.connect("clicked", self.on_exit_to_console)

        table = gtk.Table(rows=4, columns=3, homogeneous=False)
        table.set_border_width(0)
        table.attach(gtk.Label(''), 0, 1, 0, 1, xoptions=gtk.EXPAND)
        table.attach(button1, 1, 2, 0, 1)
        table.attach(button2, 1, 2, 1, 2)
        table.attach(button3, 1, 2, 2, 3)
        table.attach(button4, 1, 2, 3, 4)
        table.attach(gtk.Label(''), 2, 3, 0, 1, xoptions=gtk.EXPAND)
        page.pack_start(table, expand=False)

        nav_bar = self._create_nav_bar(prev_page="welcome")
        page.pack_end(nav_bar, expand=False)
        return page

    def _create_reconfigure_page(self):
        page = self._create_page("How would you like to reconfigure your display?")

        hbox = gtk.HBox()

        button1 = self._create_button("Reset to defaults",
                                      "Use default (generic) configuration")
        button1.connect("clicked", self.on_reset_to_default_xorg_conf)
        button2 = self._create_button("Generate xorg.conf",
                                      "Creates a new configuration for this hardware")
        button2.connect("clicked", self.on_generate_xorg_conf)
        button3 = self._create_button("Restore backup",
                                      "Use your backed-up configuration")
        button3.connect("clicked", self.on_restore_backup)

        table = gtk.Table(rows=3, columns=3, homogeneous=False)
        table.set_border_width(0)
        table.attach(gtk.Label(''), 0, 1, 0, 1, xoptions=gtk.EXPAND)
        table.attach(button1, 1, 2, 0, 1)
        table.attach(button2, 1, 2, 1, 2)
        table.attach(button3, 1, 2, 2, 3)
        table.attach(gtk.Label(''), 2, 3, 0, 1, xoptions=gtk.EXPAND)
        page.pack_start(table, expand=False)

        nav_bar = self._create_nav_bar(prev_page="actions")
        page.pack_end(nav_bar, expand=False)
        return page

    def _create_troubleshoot_page(self):
        page = self._create_page("What information would you like to review?")

        hbox = gtk.HBox()

        button1 = self._create_button("X server log",
                                      "Review the failed session Xorg.0.log")
        button1.connect("clicked", self.on_view_file, "/var/log/Xorg.0.log")
        button2 = self._create_button("lightdm's X startup log",
                                      "Review the display manager log")
        button2.connect("clicked", self.on_view_file, "/var/log/lightdm/x-0.log")
        button3 = self._create_button("Edit X Config",
                                      "Edit xorg.conf configuration file")
        button3.connect("clicked", self.on_edit_file, "/etc/X11/xorg.conf")
        button4 = self._create_button("Archive",
                                      "Package up configuration and logs for sharing")
        button4.connect("clicked", self.on_make_archive)

        table = gtk.Table(rows=4, columns=3, homogeneous=False)
        table.set_border_width(0)
        table.attach(gtk.Label(''), 0, 1, 0, 1, xoptions=gtk.EXPAND)
        table.attach(button1, 1, 2, 0, 1)
        table.attach(button2, 1, 2, 1, 2)
        table.attach(button3, 1, 2, 2, 3)
        table.attach(button4, 1, 2, 3, 4)
        table.attach(gtk.Label(''), 2, 3, 0, 1, xoptions=gtk.EXPAND)
        page.pack_start(table, expand=False)

        nav_bar = self._create_nav_bar(prev_page="actions")
        page.pack_end(nav_bar, expand=False)
        return page

    def has_backup(self):
        if (os.path.exists(self.xorg_conf_backup_path) or
            os.path.exists(self.xorg_conf_d_backup_path)):
            return True
        return False

    def backup_xorg_conf(self):
        '''Backs up xorg.conf and xorg.conf.d, overwriting any pre-existing backups'''
        if os.path.exists(self.xorg_conf_path):
            if os.path.exists(self.xorg_conf_backup_path):
                shutil.rm(self.xorg_conf_backup_path)
            shutil.copy(self.xorg_conf_path, self.xorg_conf_backup_path)
        if os.path.exists(self.xorg_conf_d_path):
            if os.path.exists(self.xorg_conf_d_backup_path):
                shutil.rmtree(self.xorg_conf_d_backup_path)
            shutil.copytree(self.xorg_conf_d_path, self.xorg_conf_d_backup_path)

    def update_frame(self):
        self.window.ensure_style()
        style = self.window.get_style()
        color = style.bg[gtk.STATE_SELECTED]
        self.frame.modify_bg(gtk.STATE_NORMAL, color)

        self.page_title.modify_fg(gtk.STATE_NORMAL, color)

    def on_page(self, widget, page_name):
        new_page = self.pages[page_name]
        if self.current_page is not None:
            self.frame.remove(self.current_page)
        if page_name == "welcome":
            self.page_title.set_markup("<big>Failsafe-X</big>")
        else:
            self.page_title.set_markup("<big>Failsafe-X: %s</big>" %(page_name.capitalize()))
        self.frame.add(new_page)
        self.current_page = new_page
        self.frame.show_all()

    def on_low_res_mode(self, widget):
        pass

    def on_exit_to_console(self, widget):
        pass

    def on_reset_to_default_xorg_conf(self, widget):
        if not self.backup_xorg_conf():
            dbg("Error: Could not backup the config")
            return
        shutils.rm(self.xorg_conf)
        shutils.rmtree(self.xorg_conf_d)

    def on_generate_xorg_conf(self, widget):
        if not self.backup_xorg_conf():
            dbg("Error: Could not backup the config")
            return
        pass

    def on_restore_backup(self, widget):
        pass

    def on_view_file(self, widget, filename):
        execute("zenity --text-info --filename=%s --width=640 --height=480" %(filename))

    def on_edit_file(self, widget, filename):
        if not os.path.exists(self.xorg_conf_path):
            # TODO: Stub in an xorg.conf
            pass
        execute("zenity --text-info --editable --filename=%s --width=640 --height=480" %(self.xorg_conf_path))

    def on_make_archive(self, widget):
        xorg_backup_name = "failsafeX-backup-${timestamp}"
        xorg_backup_dir  = $(mktemp -d -t ${xorg_backup_name}.XXX)
        xorg_backup_file = "/var/log/%s.tar" %(xorg_backup_name)

        shutils.copy("/etc/X11/xorg.conf", xorg_backup_dir)
        shutils.copy("/var/log/Xorg.0.log", xorg_backup_dir)
        shutils.copy("/var/log/Xorg.0.log.old", xorg_backup_dir)
        shutils.copytree("/var/log/gdm", xorg_backup_dir)
        shutils.copytree("/var/log/lightdm", xorg_backup_dir)
        execute("lspci -vvnn > %s/lspci-vvnn.txt" %(xorg_backup_dir))
        execute("xrandr --verbose > %s/xrandr-verbose.txt" %(xorg_backup_dir))
        execute("tar -cf %s %s" %(xorg_backup_file, xorg_backup_dir))
        shutils.rmtree(xorg_backup_dir)


    def on_delete(self, widget, event):
        gtk.main_quit()

    def on_close(self, widget):
        gtk.main_quit()


if __name__ == "__main__":
    dbg("main:  Starting program")
    app = XDiagnoseApplication()
    gtk.main()

    dbg("main:  Ending program")
    sys.exit(0)


